@@ -181,6 +181,8 @@ smtp_address_add_detail_temp(const struct smtp_address *address,
 
 int smtp_address_cmp(const struct smtp_address *address1,
 		     const struct smtp_address *address2) ATTR_NULL(1, 2);
+int smtp_address_cmp_icase(const struct smtp_address *address1,
+			  const struct smtp_address *address2) ATTR_NULL(1, 2);
 
 static inline bool ATTR_NULL(1, 2)
 smtp_address_equals(const struct smtp_address *address1,
@@ -188,18 +190,25 @@ smtp_address_equals(const struct smtp_address *address1,
 {
 	return (smtp_address_cmp(address1, address2) == 0);
 }
+static inline bool ATTR_NULL(1, 2)
+smtp_address_equals_icase(const struct smtp_address *address1,
+			  const struct smtp_address *address2)
+{
+	return (smtp_address_cmp_icase(address1, address2) == 0);
+}
 
 static inline bool ATTR_NULL(1) ATTR_PURE
 smtp_address_isnull(const struct smtp_address *address)
 {
-	return (address == NULL || address->localpart == NULL);
+	return (address == NULL || address->localpart == NULL ||
+		*address->localpart == '\0');
 }
 
 static inline bool ATTR_NULL(1) ATTR_PURE
 smtp_address_is_broken(const struct smtp_address *address)
 {
 	return (address != NULL &&
-		smtp_address_isnull(address) &&
+		(address->localpart == NULL || *address->localpart == '\0') &&
 		(address->raw != NULL && *address->raw != '\0'));
 }
 
